WoWJapanizer = LibStub("AceAddon-3.0"):NewAddon("WoWJapanizer", "AceConsole-3.0")

WoWJapanizer.FONT = "Interface\\AddOns\\WoWJapanizer\\font\\ipagui.ttf";
WoWJapanizer.DEBUG = false -- false when release

WoWJapanizer.property = {}

function WoWJapanizer:OnInitialize()
    self.version = GetAddOnMetadata("WoWJapanizer", "Version")
    print(string.format("Welcome to WoWJapanizer Ver: %s.\nSetting is /cj or /WoWJapanizer.", self.version))

    self.db = LibStub('AceDB-3.0'):New("WoWJapanizerDB")
	self.db:RegisterDefaults({
        profile = {
            quest       = { questlog = true, worldmap = true, gossip = true, questlog_movable = false },
            item        = { tooltip = true },
            spell       = { tooltip = true, buff = true },
            achievement = { tooltip = true, advice = true },
            developer   = self.DEBUG, -- false when release
            development = { debugger = self.DEBUG }, -- false when release
        }
    })

    -- Quest --
    WoWJapanizerQuestLog:OnInitialize()
    WoWJapanizerWorldMapQuest:OnInitialize()
    WoWJapanizerQuestGossip:OnInitialize()

    -- ToolTip --
    WoWJapanizerGameToolTip:OnInitialize()
    WoWJapanizerItemRefTooltip:OnInitialize()

    WoWJapanizerBuffToolTip:OnInitialize()
    WoWJapanizerGlyphToolTip:OnInitialize()
    WoWJapanizerGarrisonToolTip:OnInitialize()

end

function WoWJapanizer:OnEnable()
    self:DebugLog("WoWJapanizer: OnEnable.");

    -- Option --
    self:SetupOptions()

    -- Quest --
    WoWJapanizerQuestLog:OnEnable()
    WoWJapanizerWorldMapQuest:OnEnable()
    WoWJapanizerQuestGossip:OnEnable()

    -- ToolTip --
    WoWJapanizerGameToolTip:OnEnable()
    WoWJapanizerItemRefTooltip:OnEnable()

	-- Buff	
    WoWJapanizerBuffToolTip:OnEnable()
	-- Glyph
    WoWJapanizerGlyphToolTip:OnEnable()
	
	WoWJapanizerGarrisonToolTip:OnEnable()
end

function WoWJapanizer:OnDisable()
    self:DebugLog("WoWJapanizer: OnDisable.");

    -- Quest --
    WoWJapanizerQuestLog:OnDisable()
    WoWJapanizerWorldMapQuest:OnDisable()
    WoWJapanizerQuestGossip:OnDisable()

    -- ToolTip --
    WoWJapanizerGameToolTip:OnDisable()
    WoWJapanizerItemRefTooltip:OnDisable()

	-- Buff	
    WoWJapanizerBuffToolTip:OnDisable()

	-- Glyph
    WoWJapanizerGlyphToolTip:OnDisable()
	
	WoWJapanizerGarrisonToolTip:OnDisable()

end

function WoWJapanizer:LoadAddOn(addon)
    return true
end

function WoWJapanizer:uc(str)
    return string.gsub(str, "(%w)", function(s)
        return string.upper(s)
    end, 1)
end

function WoWJapanizer:lc(str)
    return string.gsub(str, "(%w)", function(s)
        return string.lower(s)
    end, 1)
end

function WoWJapanizer:SetupOptions()
    function GetOptions()
        return {
            type = "group",
            name = "WoWJapanizer",
            args = {
                general = {
                    type = "group",
                    name = "WoWJapanizer",
                    args = {
                        desc = {
                            type = "description",
                            name = "WoWJapanizer is addon that displays the Japanese translation of Quest information and Spell/Item ToolTips.",
                            order = 1,
                        },
                    },
                },
            },
        }
    end

    LibStub("AceConfig-3.0"):RegisterOptionsTable("WoWJapanizer", GetOptions())

    local dialog =  LibStub("AceConfigDialog-3.0")
    dialog:AddToBlizOptions("WoWJapanizer", nil, nil, "general")
    dialog:AddToBlizOptions("WoWJapanizer", "Quest", "WoWJapanizer", "quest")
    dialog:AddToBlizOptions("WoWJapanizer", "ToolTip", "WoWJapanizer", "tooltip")
    dialog:AddToBlizOptions("WoWJapanizer", "Developer", "WoWJapanizer", "developer")

    self:RegisterChatCommand("cj", "ChatCommand")
    self:RegisterChatCommand("WoWJapanizer", "ChatCommand")
end

function WoWJapanizer:ChatCommand(input)
    if input == 'debug' then
        -- Debug Window --
        if self.db.profile.development.debugger then
            WoWJapanizerDebugFrame:Show()
        end
    else
        self:OpenConfigDialog()
    end
end

function WoWJapanizer:OpenConfigDialog()
    LibStub("AceConfigDialog-3.0"):Open("WoWJapanizer")
end

function WoWJapanizer:DebugLog(args)
    if not self.DEBUG then
        return
    end

    if type(args) == 'string' then
        self:Print(args)
        return
    end

    if type(args) == 'number' then
        self:Print(tostring(args))
        return
    end

    if type(args) == 'table' then
        self:Print("----------")
        for n, v in pairs(args) do
            self:Print(n .. ' : ' .. v)
        end
        self:Print("----------")
        return
    end
end
